import ij.*;
import ij.process.*;
import ij.plugin.filter.*;
import java.util.*;

import java.awt.Color;
import speckles.Speckle;
import speckles.SpeckleWriter;


/**
 * Draws speckles on the selected image.  Draws the complete speckle track
 * on every frame that it exists.
 *
 */
public class Draw_Speckles implements PlugInFilter{
        ImagePlus implus;

    /**
     *
     * @param args - not used.
     * @param implus - a copy will be made and drawn upon.
     * @return does all, can be used for any image type.
     */
    public int setup(String args, ImagePlus implus){

            this.implus = implus;
            return DOES_ALL;

    }

    /**
     *
     * @param imp not used.
     */
    public void run (ImageProcessor imp){
        HashSet<Speckle> speckles = SpeckleWriter.readCSVSpeckles(IJ.getInstance());
        
        HashMap<Speckle, Color> colormap = new HashMap<Speckle, Color>();
        int j = 0;
        for(Speckle s: speckles){
            
            int r = ((100*j + 255)%256)<<16;
            int g = ((13*j + 255)%256)<<8;
            int b = (50*j + 205)%256;
            
            colormap.put(s,new Color(r+g+b));
            
            j++;
        }
        ImageStack istack = implus.getStack();
        ImageStack outstack = new ImageStack(implus.getWidth(), implus.getHeight());
        for(int i = 1; i<=istack.getSize(); i++){
                            
            ImageProcessor ip = istack.getProcessor(i).convertToRGB().duplicate();
            
            for(Speckle speck: speckles){
            
                if(speck.exists(i))
                    drawTrack(ip,speck, colormap.get(speck));
            
            }
            outstack.addSlice("next", ip);
        }
            ImagePlus x = new ImagePlus("corelated",outstack);
            x.show();
        
    }

    /**
     *  Draws the speckle track speck, with color c onto ip.  Consecutive
     *  marks are drawn by connecting lines.
     *
     *
     * @param ip - image that will be drawn on.
     * @param speck - speckle track that is to be drawn.
     * @param c - the color of the track
     */
    public void drawTrack(ImageProcessor ip, Speckle speck, Color c){
        double[] current=null, last=null;
        ip.setColor(c);
        for(Integer i: speck){
            current = speck.getCoordinates(i);
            if(last!=null)
                ip.drawLine((int)current[0], (int)current[1], (int)last[0], (int)last[1]);
            last = current;
            
            
        }
        
        
        ip.drawDot((int)current[0], (int)current[1]);
        
    }
  
}
